/*********************************************************************************************************
* Software License Agreement (BSD License)                                                               *
* Author: Sebastien Decugis <sdecugis@nict.go.jp>							 *
*													 *
* Copyright (c) 2008, WIDE Project and NICT								 *
* All rights reserved.											 *
* 													 *
* Redistribution and use of this software in source and binary forms, with or without modification, are  *
* permitted provided that the following conditions are met:						 *
* 													 *
* * Redistributions of source code must retain the above 						 *
*   copyright notice, this list of conditions and the 							 *
*   following disclaimer.										 *
*    													 *
* * Redistributions in binary form must reproduce the above 						 *
*   copyright notice, this list of conditions and the 							 *
*   following disclaimer in the documentation and/or other						 *
*   materials provided with the distribution.								 *
* 													 *
* * Neither the name of the WIDE Project or NICT nor the 						 *
*   names of its contributors may be used to endorse or 						 *
*   promote products derived from this software without 						 *
*   specific prior written permission of WIDE Project and 						 *
*   NICT.												 *
* 													 *
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED *
* WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A *
* PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR *
* ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 	 *
* LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 	 *
* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR *
* TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF   *
* ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.								 *
*********************************************************************************************************/

#include "tests.h"

/* Test for (some functions of) the peer module */

/* Additional header for accessing internal peer module data */
#include <peer-internal.h>

static int cb_called = 0;

static int cb( void * data, struct _pa_ * pa)
{
	CHECK( data, (void *)&cb_called );
	cb_called = 1;
	return -1; /* delete the peer */
}

/* Main test routine */
int main(int argc, char *argv[])
{
	_peer_t * peer;
	char * p1 = strdup("p1.localhost");
	char * p2 = strdup("p2.localhost");
	char * p3 = strdup("p3.localhost");
	char * p4 = strdup("p4.localhost");
	char * p5 = strdup("p5.localhost");
	char * realm = strdup("localhost");
	
	/* First, initialize the daemon modules */
	INIT_WAAAD();
	
	/* Test creating new peer objects */
	{
		peer_add_t data;
		
		memset(&data, 0, sizeof(data));
		
		CHECK( EINVAL, peer_add( p1, strlen(p1), &data ) );
		
		data.pa_proto = IPPROTO_TCP;
		CHECK( 0, peer_add( p1, strlen(p1), &data ) );
		
		data.pa_proto = IPPROTO_SCTP;
		data.pa_streams = 20;
		CHECK( 0, peer_add( p2, strlen(p2), &data ) );
		
		data.pa_flags = PA_EXPIRY;
		CHECK( 0, peer_add( p3, strlen(p3), &data ) );
		
		data.pa_flags = PA_EXPIRY | PA_CB;
		data.pa_cbparm = &cb_called;
		data.pa_cb = cb;
		CHECK( 0, peer_add( p4, strlen(p4), &data ) );
		
		/* Now check these objects were created */
		CHECK( 4, peer_dump_hash(FULL) );
	}
	
	/* Search a peer */
	{
		uti_list_t * l = NULL;
		
		/* Test when not found */
		CHECK( ENOENT, _peer_struct_search(p5, 0, NULL, &l)  );
		CHECK( 0, l == NULL ? 1 : 0 );
		CHECK( 0, pthread_mutex_unlock( &((_peer_hash_t *)(l->head))->lock )  );
		
		/* Test when found */
		CHECK( 0, _peer_struct_search(p1, 0, NULL, &l)  );
		peer = _P(l->o);
		CHECK( 0, pthread_mutex_unlock( &((_peer_hash_t *)(l->head))->lock )  );
	}
	
	/* peer_get */
	{
		void * data = NULL;
		
		CHECK( 0, peer_get( (peer_t *)peer, PEER_DIAMETER_ID, &data ) );
		CHECK( 0, strcmp( (char *)data, p1 ) );
		
		CHECK( 0, peer_get( (peer_t *)peer, PEER_REALM, &data ) );
		CHECK( 0, strcmp( (char *)data, realm ) );
		
		CHECK( 0, peer_get( (peer_t *)peer, PEER_ONLINE, &data ) );
		CHECK( 0, data ? 1 : 0 );
	}
	
	/* Delete a peer */
	{
		CHECK( 0, _peer_struct_unlink( peer ) );
		CHECK( 0, _peer_struct_destroy( &peer ) );
		CHECK( 3, peer_dump_hash(FULL) );
	}
		
	
	/* how to test all the remaining? ... maybe too complex for this testsuite... */
	
	free(p1);
	free(p2);
	free(p3);
	free(p4);
	free(p5);
	free(realm);
	
	/* That's all for the tests yet */
	PASSTEST();
} 
	
